'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _queryBuilder = require('./queryBuilder');

var _relay = require('./relay');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Node class
 */

var Node = function () {
  function Node(_ref) {
    var model = _ref.model;
    var _ref$tree = _ref.tree;
    var tree = _ref$tree === undefined ? {} : _ref$tree;
    var _ref$related = _ref.related;
    var related = _ref$related === undefined ? undefined : _ref$related;
    var _ref$args = _ref.args;
    var args = _ref$args === undefined ? {} : _ref$args;
    var _ref$connection = _ref.connection;
    var connection = _ref$connection === undefined ? {} : _ref$connection;
    var _ref$name = _ref.name;
    var name = _ref$name === undefined ? '' : _ref$name;

    _classCallCheck(this, Node);

    (0, _assert2.default)(model, 'You need to provide a thinky Model');

    this.model = model;
    this.related = related;
    this.tree = tree;
    this.args = args;
    this.connection = connection;
    this.name = name; // The name will be populated based to AST name if not provided
  }

  _createClass(Node, [{
    key: 'query',
    value: function () {
      var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee(thinky) {
        var Query, queryResult;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                this.args.relations = this.tree;
                Query = (0, _queryBuilder.buildQuery)(this.model, this.args, thinky);
                queryResult = void 0;

                if (!this.args.list) {
                  _context.next = 9;
                  break;
                }

                _context.next = 6;
                return Query.run();

              case 6:
                queryResult = _context.sent;
                _context.next = 12;
                break;

              case 9:
                _context.next = 11;
                return Query.nth(0).default(null).run();

              case 11:
                queryResult = _context.sent;

              case 12:
                return _context.abrupt('return', queryResult);

              case 13:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function query(_x) {
        return ref.apply(this, arguments);
      }

      return query;
    }()

    /**
     * Resolve from tree
     *
     * @param source
     * @returns {*}
     */

  }, {
    key: 'resolve',
    value: function () {
      var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee2(source) {
        var result;
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                result = source[this.name];
                return _context2.abrupt('return', result);

              case 2:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function resolve(_x2) {
        return ref.apply(this, arguments);
      }

      return resolve;
    }()

    /**
     * Create a relay connection
     *
     * @returns {{connectionType, edgeType, nodeType, resolveEdge, connectionArgs, resolve}|*}
     */

  }, {
    key: 'connect',
    value: function connect() {
      /*eslint-disable */
      if (!this.connection.name) throw new Error("Please specify a connection name, before call connect on a Node");
      if (!this.connection.type) throw new Error("Please specify a connection type, before call connect on a Node");
      /*eslint-enable */

      return (0, _relay.resolveConnection)(this);
    }

    /**
     * Generate data tree
     *
     * @param treeSource
     * @param thinky
     * @returns {Array}
     */

  }, {
    key: 'generateDataTree',
    value: function () {
      var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee3(treeSource, thinky) {
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                if (this.isRelated()) {
                  _context3.next = 6;
                  break;
                }

                _context3.next = 3;
                return this.query(thinky);

              case 3:
                treeSource = _context3.sent;
                _context3.next = 10;
                break;

              case 6:
                if (!(this.isRelated() && treeSource)) {
                  _context3.next = 10;
                  break;
                }

                _context3.next = 9;
                return this.resolve(treeSource);

              case 9:
                treeSource = _context3.sent;

              case 10:
                return _context3.abrupt('return', treeSource);

              case 11:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function generateDataTree(_x3, _x4) {
        return ref.apply(this, arguments);
      }

      return generateDataTree;
    }()

    /**
     * Set Relation Tree.
     * the three is an array of nodes
     *
     * @param tree array
     */

  }, {
    key: 'setTree',
    value: function setTree(tree) {
      this.tree = tree;
    }

    /**
     * Append Nodes to tree
     *
     * @param Node
     */

  }, {
    key: 'appendToTree',
    value: function appendToTree(Node) {
      this.tree = _extends({}, this.tree, Node);
    }

    /**
     * Get tree
     *
     * @returns {*}
     */

  }, {
    key: 'getTree',
    value: function getTree() {
      return this.tree;
    }

    /**
     * Set args
     *
     * @param args
     */

  }, {
    key: 'appendArgs',
    value: function appendArgs(args) {
      this.args = _extends({}, this.args, args);
    }

    /**
     * Determine if this node is a connection
     *
     * @returns {string|*}
     */

  }, {
    key: 'isConnection',
    value: function isConnection() {
      return this.connection.name && this.connection.type;
    }

    /**
     * Determine if the node is related
     *
     * @returns {boolean}
     */

  }, {
    key: 'isRelated',
    value: function isRelated() {
      return Boolean(this.related);
    }

    /**
     * Get model of the Node
     *
     * @returns {*}
     */

  }, {
    key: 'getModel',
    value: function getModel() {
      return this.model;
    }

    /**
     * Get model Name
     *
     * @return {string}
     */

  }, {
    key: 'getModelName',
    value: function getModelName() {
      return this.model.getTableName();
    }
  }]);

  return Node;
}();

exports.default = Node;