'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = generateTree;

var _thinkyExportSchema = require('thinky-export-schema');

var _thinkyExportSchema2 = _interopRequireDefault(_thinkyExportSchema);

var _graphql = require('graphql');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _node = require('./node');

var _node2 = _interopRequireDefault(_node);

var _queryBuilder = require('./queryBuilder');

var _relay = require('./relay');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function inList(list, attribute) {
  return list.indexOf(attribute) !== -1;
}

/**
 * Create a tree
 *
 * @param simpleAST
 * @param type
 * @param context
 * @returns {{}}
 */
function generateTree(simpleAST, type, context) {
  var result = {};

  type = type.ofType || type;

  Object.keys(simpleAST.fields).forEach(function (key) {
    var fieldAST = simpleAST.fields[key],
        name = fieldAST.key || key,
        // eslint-disable-line prefer-const
    fieldType = type._fields[name] && type._fields[name].type;

    var args = fieldAST.args;
    var includeResolver = type._fields[name].resolve;

    // No point continue is no resolve or $Node not found on the resolver
    if (!includeResolver || includeResolver && !includeResolver.$Node) {
      return;
    }

    if ((0, _relay.isConnection)(fieldType)) {
      fieldAST = (0, _relay.nodeAST)(fieldAST);
      fieldType = (0, _relay.nodeType)(fieldType);
    }

    // No point inncluding if no fields have been asked for
    if (!fieldAST) {
      return;
    }

    var resolverNode = includeResolver.$Node,
        params = {};

    Object.keys(resolverNode).forEach(function (key) {
      params[key] = resolverNode[key];
    });

    // To Prevent further Circular
    // reference i'll re-construct the object
    // on the tree
    var Node = new _node2.default(params);
    Node.name = Node.name || name;

    var includeOptions = (0, _queryBuilder.argsToFindOptions)(args, Node.getModel());

    if (Node.isRelated()) {
      var Related = Node.related;
      var Model = Related.model;
      var modelSchema = (0, _thinkyExportSchema2.default)(Model);
      var allowedAttributes = Object.keys(modelSchema.fields);

      includeOptions.attributes = (includeOptions.attributes || []).concat(Object.keys(fieldAST.fields).map(function (key) {
        return fieldAST.fields[key].key || key;
      })).filter(inList.bind(null, allowedAttributes));

      if (includeResolver.$before) {
        includeOptions = includeResolver.$before(includeOptions, args, context, {
          ast: fieldAST,
          type: type
        });
      }

      if (Related.type === 'hasMany') {
        includeOptions.attributes.push(Related.rightKey);
      } else {
        includeOptions.attributes.push(Related.leftKey);
      }

      var nestedNode = generateTree(fieldAST, fieldType, context);

      var hasNestedNode = Object.keys(nestedNode).length > 0;

      if (hasNestedNode) {
        includeOptions.attributes = _lodash2.default.uniq(includeOptions.attributes);
        Node.appendToTree(nestedNode);
      }

      includeOptions.list = fieldType.typeOf || fieldType instanceof _graphql.GraphQLList;
      Node.appendArgs(includeOptions);

      result[Node.name] = Node;
    }
  });

  return result;
}