'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _resolver = require('./resolver');

var _resolver2 = _interopRequireDefault(_resolver);

var _typeMapper = require('./typeMapper');

var _typeMapper2 = _interopRequireDefault(_typeMapper);

var _nodeDefinition = require('./relay/nodeDefinition');

var _modelToGqlObjectType = require('./modelToGqlObjectType');

var _modelToGqlObjectType2 = _interopRequireDefault(_modelToGqlObjectType);

var _node = require('./node');

var _node2 = _interopRequireDefault(_node);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * GraphqlThinky
 */

var GraphqlThinky =

/**
 * Graphus constructor accept a Thinky
 * Instance
 *
 * @param thinky
 */
function GraphqlThinky(thinky) {
  var _this = this;

  _classCallCheck(this, GraphqlThinky);

  this.resolve = function (modelName, related) {
    var opts = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

    opts.thinky = _this.thinky;

    var Node = _this.node(modelName, related);
    return (0, _resolver2.default)(Node, opts);
  };

  this.connect = function (modelName, related) {
    var opts = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

    /*eslint-disable */
    if (!opts.connection) throw Error('Please provide a connection option.');
    if (!opts.connection.name) throw Error('Please provide a name for the connection based on Model: ' + modelName + '.');
    if (!opts.connection.type) throw Error('Please provide a type for the connection based on Model: ' + modelName + '.');
    /*eslint-enable */

    opts.thinky = _this.thinky;

    var NodeConnector = _this.node(modelName, related, opts).connect();

    return {
      type: NodeConnector.connectionType,
      args: _extends({}, NodeConnector.connectionArgs, opts.args || {}),
      resolve: NodeConnector.resolve
    };
  };

  this.modelToGraphQLDefinition = function (model) {
    var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

    if (typeof model === 'string') {
      model = _this.thinky.models[model];
    }

    if (!model) {
      throw new Error('Model name: ' + model + ' not found.');
    }

    return (0, _typeMapper.toGraphQLDefinition)(model, opts);
  };

  this.createModelType = function (model) {
    var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

    if (typeof model === 'string') {
      model = _this.thinky.models[model];
    }

    if (!model) {
      throw new Error('Model name: ' + model + ' not found.');
    }

    // Add node interface if relay is specified
    if (opts.globalId === true) {
      opts.interfaces = [_this.nodeInterface];
    }

    var gqlObjectType = (0, _modelToGqlObjectType2.default)(model, opts);

    _this.nodeTypeMapper.mapTypes(_defineProperty({}, model._schema._model._name, gqlObjectType));

    return gqlObjectType;
  };

  this.node = function (modelName, related) {
    var opts = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

    if (!opts.name) {
      opts.name = '';
    }

    var connection = opts.connection || {};

    var models = _this.thinky.models;

    var modelTarget = models[modelName],
        relation = related;

    if (!modelTarget) {
      throw new Error('Model ' + modelName + ' not found.');
    }

    // Relation is specified as a string
    if (typeof related === 'string') {
      relation = modelTarget._joins[related];

      // relation not found can't continue
      if (!relation) {
        throw new Error('Tried to access relation ' + related + ' of the Model ' + modelName + ',\n             but relation not found.');
      }
    }

    modelTarget = relation ? relation.model : modelTarget;

    return new _node2.default({
      name: related || opts.name,
      model: modelTarget,
      related: relation,
      connection: _extends({}, connection)
    });
  };

  this.thinky = thinky;

  var nodeMapper = (0, _nodeDefinition.nodeInterfaceMapper)(thinky.models);

  this.nodeField = nodeMapper.nodeField;
  this.nodeInterface = nodeMapper.nodeInterface;
  this.nodeTypeMapper = nodeMapper.nodeTypeMapper;
}

/**
 * Resolve a node
 *
 * @param modelName
 * @param related
 * @param opts
 * @returns {Resolver}
 */


/**
 * Create a relay connection
 *
 * @param modelName
 * @param related
 * @param opts
 * @returns {Resolver}
 */


/**
 * return a graphQL Object Type
 * definition from a Thinky Model.
 *
 * @param model
 * @param opts
 */


/**
 * Create a GraphQLObjectType based
 * to a thinky model. Using this helper
 * you'll get free Node type mapping
 * for relay
 *
 * @param model
 * @param opts
 * @returns {GraphQLObjectType}
 */


/**
 * Create a node
 *
 * @param modelName
 * @param related
 * @returns {Node}
 */
;

GraphqlThinky.typeMapper = _typeMapper2.default;
exports.default = GraphqlThinky;