'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.argsToFindOptions = argsToFindOptions;
exports.resolveJoin = resolveJoin;
exports.buildQuery = buildQuery;

var _thinkyExportSchema = require('thinky-export-schema');

var _thinkyExportSchema2 = _interopRequireDefault(_thinkyExportSchema);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Args to find options
 *
 * @param args
 * @param modelSchama
 * @returns {{}}
 */
function argsToFindOptions(args, model) {
  var result = {
    filter: {},
    limit: undefined,
    skip: undefined,
    order: undefined
  },
      modelSchema = (0, _thinkyExportSchema2.default)(model),
      attributes = Object.keys(modelSchema.fields).concat('id');

  if (args) {
    Object.keys(args).forEach(function (key) {
      if (attributes.indexOf(key) !== -1) {
        result.filter = result.filter || {};
        result.filter[key] = args[key];
      }

      if (key === 'limit' && args[key]) {
        result.limit = parseInt(args[key], 10);
      }

      if (key === 'skip' && args[key]) {
        result.skip = parseInt(args[key], 10);
      }

      if (key === 'order' && args[key]) {
        if (args[key].indexOf('reverse:') === 0) {
          result.order = [args[key].substring(8), 'DESC'];
        } else {
          result.order = [args[key], 'ASC'];
        }
      }
    });

    return result;
  }
}

/**
 * Resolve join for nested relations
 * recursivily
 *
 * @param thinky
 * @param node
 * @returns {*}
 */
function resolveJoin(thinky, node) {
  if (!node.related) {
    return;
  }

  var _node$related = node.related;
  var model = _node$related.model;
  var type = _node$related.type;
  var attributes = node.args.attributes;


  var resolvedAttributes = {};
  var relationsResolved = {};
  var modelSchema = (0, _thinkyExportSchema2.default)(model);

  Object.keys(node.tree).forEach(function (relatedKey) {
    var resolvedJoin = resolveJoin( // ** Recursion
    thinky, node.tree[relatedKey]);

    if (resolvedJoin) {
      relationsResolved[relatedKey] = resolvedJoin;
    }
  });

  attributes.forEach(function (attribute) {
    if (!node.tree.hasOwnProperty(attribute)) {
      resolvedAttributes[attribute] = true;
    }
  });

  // Add the id
  resolvedAttributes.id = true;

  return _extends({
    _apply: function _apply(seq) {
      var findArgs = _extends({}, node.args, {
        attributes: attributes
      });

      var columns = {};

      for (var field in resolvedAttributes) {
        if (!modelSchema.relationships.hasOwnProperty(field)) {
          columns[field] = resolvedAttributes[field];
        }
      }

      // Special case with belongs to,
      // we can't chain any sequence, need to open a issue
      // on thinky ex: pluck can't be chained
      if (type === 'belongsTo') {
        findArgs.attributes = false;
        findArgs.order = false;
      } else {
        findArgs.attributes = columns;
      }

      return buildQuery(seq, findArgs, thinky);
    }
  }, relationsResolved);
}

/**
 * Build query
 *
 * @param seq
 * @param args
 * @param thinky
 * @param options
 * @returns {*}
 */
function buildQuery(seq, args, thinky) {
  args.relations = args.relations || {};

  var Query = seq;

  if (_lodash2.default.isArray(args.attributes)) {
    Query = seq.withFields(args.attributes);
  } else if (_lodash2.default.isObject(args.attributes)) {
    Query = seq.withFields(args.attributes);
  }

  if (args.filter && Object.keys(args.filter).length > 0) {
    Object.keys(args.filter).forEach(function (fieldName) {
      if (_lodash2.default.isFunction(args.filter[fieldName])) {
        Query = Query.filter(args.filter[fieldName]);
        delete args.filter[fieldName];
      }
    });

    Query = Query.filter(args.filter);
  }

  if (args.count) {
    var countQuery = Query.merge(function () {
      return {
        fullCount: Query._query.count()
      };
    });

    Query = countQuery;
  }

  if (args.order && args.order[1] === 'DESC') {
    Query = Query.orderBy(thinky.r.desc(args.order[0]));
  } else if (args.order && args.order[0]) {
    Query = Query.orderBy(args.order[0]);
  }

  if (args.offset) {
    Query = Query.skip(parseInt(args.offset, 10));
  }

  if (args.limit) {
    Query = Query.limit(parseInt(args.limit, 10));
  }

  var joinRelations = {};

  // Compose the object which will join
  // the results, recursively
  Object.keys(args.relations).forEach(function (relation) {
    // Extract requested fields for the nested relation
    var resolvedJoin = resolveJoin(thinky, args.relations[relation]);

    if (resolvedJoin) {
      joinRelations[relation] = resolvedJoin;
    }
  });

  if (Object.keys(joinRelations).length > 0) {
    Query = Query.getJoin(joinRelations);
  }

  return Query;
}