'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = relateIncludes;

var _Node = require('./Node');

var _Node2 = _interopRequireDefault(_Node);

var _thinkyExportSchema = require('thinky-export-schema');

var _thinkyExportSchema2 = _interopRequireDefault(_thinkyExportSchema);

var _queryBuilder = require('./queryBuilder');

var _relay = require('./relay');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _graphql = require('graphql');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function inList(list, attribute) {
  return ~list.indexOf(attribute);
}

function relateIncludes(simpleAST, type, context) {

  var result = {}; //{relations: {}, attributes: [], order: []};

  type = type.ofType || type;

  Object.keys(simpleAST.fields).forEach(function (key) {

    var fieldAST = simpleAST.fields[key],
        name = fieldAST.key || key,
        fieldType = type._fields[name] && type._fields[name].type,
        args = fieldAST.args,
        includeResolver = type._fields[name].resolve;

    // No point continue is no resolve or $Node not found on the resolver
    if (!includeResolver || includeResolver && !includeResolver.$Node) return;

    if ((0, _relay.isConnection)(fieldType)) {
      fieldAST = (0, _relay.nodeAST)(fieldAST);
      fieldType = (0, _relay.nodeType)(fieldType);
    }

    // No point inncluding if no fields have been asked for
    if (!fieldAST) return;

    var Node = includeResolver.$Node;
    var includeOptions = (0, _queryBuilder.argsToFindOptions)(args, Node.getModel());

    if (Node.isRelated()) {
      (function () {

        var Related = Node.related;
        var Model = Related.model;
        var modelSchema = (0, _thinkyExportSchema2.default)(Model);
        var allowedAttributes = Object.keys(modelSchema.fields);

        includeOptions.attributes = (includeOptions.attributes || []).concat(Object.keys(fieldAST.fields).map(function (key) {
          return fieldAST.fields[key].key || key;
        })).filter(inList.bind(null, allowedAttributes));

        if (includeResolver.$before) {

          includeOptions = includeResolver.$before(includeOptions, args, context, {
            ast: fieldAST,
            type: type
          });
        }

        includeOptions.attributes.push(Related.leftKey);

        if (Related.type === 'hasMany') {
          includeOptions.attributes.push(Related.rightKey);
        }

        if (includeOptions.order) {

          result.order = (result.order || []).concat(includeOptions.order);
          delete includeOptions.order;
        }

        var nestedNode = relateIncludes(fieldAST, fieldType, context, includeResolver.$options);

        var hasNestedNode = Object.keys(nestedNode).length > 0;

        if (hasNestedNode) {
          Object.keys(nestedNode).forEach(function (node) {
            includeOptions.attributes = includeOptions.attributes.concat(nestedNode[node].args.attributes);
          });

          includeOptions.attributes = _lodash2.default.uniq(includeOptions.attributes);
          Node.appendToTree(nestedNode);
        }

        includeOptions.list = fieldType.typeOf || fieldType instanceof _graphql.GraphQLList;
        Node.setArgs(includeOptions);

        result[Node.name] = Node;
      })();
    }
  });

  return result;
};