'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.resolveConnection = undefined;
exports.isConnection = isConnection;
exports.handleConnection = handleConnection;
exports.nodeAST = nodeAST;
exports.nodeType = nodeType;

var _graphqlRelay = require('graphql-relay');

var _resolver = require('./resolver');

var _resolver2 = _interopRequireDefault(_resolver);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Check if the type is a connection
 *
 * @param type
 * @returns {boolean|*}
 */
function isConnection(type) {
  return typeof type.name !== 'undefined' && type.name.endsWith('Connection');
}

/**
 * Handle a connecton from array
 *
 * @param values
 * @param args
 */
function handleConnection(values, args) {
  return (0, _graphqlRelay.connectionFromArray)(values, args);
}

/**
 * Node ast
 *
 * @param connectionAST
 * @returns {*|connectionFields.edges|{type, description}|string|Function}
 */
function nodeAST(connectionAST) {
  return connectionAST.fields.edges && connectionAST.fields.edges.fields.node;
}

/**
 * Get node type
 *
 * @param connectionType
 * @returns {*}
 */
function nodeType(connectionType) {
  return connectionType._fields.edges.type.ofType._fields.node.type;
}

/**
 * Export relay resolver.
 */
exports.resolveConnection = _resolver2.default;