'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _graphqlRelay = require('graphql-relay');

var _graphql = require('graphql');

var _simplifyAst = require('../simplifyAst');

var _simplifyAst2 = _interopRequireDefault(_simplifyAst);

var _base = require('./../base64.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var cursorSeparator = '$',
    cursorPrefix = 'arrayconnection' + cursorSeparator;

/**
 * Creates a cursor based on the item and the
 * index of where is located on the result set.
 * needed to identify edges
 *
 * @param item
 * @param index
 * @returns {*}
 */
function toCursor(item, index) {
  var id = item.id;
  return (0, _base.base64)(cursorPrefix + id + cursorSeparator + index);
}

/**
 * Decode a cursor into its component parts
 *
 * @param cursor
 * @returns {{id, index}}
 */
function fromCursor(cursor) {
  cursor = (0, _base.unbase64)(cursor);
  cursor = cursor.substring(cursorPrefix.length, cursor.length);

  var _cursor$split = cursor.split(cursorSeparator);

  var _cursor$split2 = _slicedToArray(_cursor$split, 2);

  var id = _cursor$split2[0];
  var index = _cursor$split2[1];


  return {
    id: id,
    index: index
  };
}

/**
 * Resolve an edge within it's
 * cursor, node and source
 *
 * @param item
 * @param index
 * @param queriedCursor
 * @param args
 * @param source
 * @returns {{cursor: *, node: *, source: *}}
 */
function resolveEdge(item, index, queriedCursor) {
  var args = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];
  var source = arguments[4];

  var startIndex = 0;
  if (queriedCursor) {
    startIndex = Number(queriedCursor.index);
  }
  if (startIndex !== 0) {
    startIndex++;
  }
  return {
    cursor: toCursor(item, index + startIndex),
    node: item,
    source: source
  };
}

/**
 * Return location information
 * of an edge
 *
 * @param resultset
 * @param limit
 * @param cursor
 * @returns {{hasMorePages: boolean, hasPreviousPage: boolean}}
 */
function createEdgeInfo(resultset, limit, cursor) {
  // retrieve full count from the first edge
  // or default 10
  var fullCount = resultset[0] && resultset[0].fullCount && parseInt(resultset[0].fullCount, 10);

  if (!resultset[0]) {
    fullCount = 0;
  }

  var hasMorePages = false;
  var hasPreviousPage = false;

  if (limit) {
    var index = cursor ? Number(cursor.index) : 0;
    var perPage = parseInt(limit, 10);
    var requested = (index + 1) * perPage;

    hasMorePages = requested < fullCount;
    hasPreviousPage = requested > perPage;
  }
  return {
    hasMorePages: hasMorePages,
    hasPreviousPage: hasPreviousPage
  };
}
/**
 * Resolve a relay connection
 *
 * @param Node
 * @returns {{connectionType, edgeType, nodeType: *, resolveEdge: resolveEdge, connectionArgs: {orderBy: {type}}, resolve: resolver}}
 */

exports.default = function (Node) {
  var connectionOpts = Node.connection,
      connectionName = connectionOpts.name,
      nodeType = connectionOpts.type,
      userParms = connectionOpts.params || {};

  connectionOpts.before = connectionOpts.before || function (options) {
    return options;
  };
  connectionOpts.after = connectionOpts.after || function (options) {
    return options;
  };

  var _connectionDefinition = (0, _graphqlRelay.connectionDefinitions)({
    nodeType: nodeType,
    name: connectionName,
    connectionFields: connectionOpts.connectionFields,
    edgeFields: connectionOpts.edgeFields
  });

  var edgeType = _connectionDefinition.edgeType;
  var connectionType = _connectionDefinition.connectionType;

  // Define the order of the connection
  // To have always a guranteed set of data
  // (if not provided)

  var orderByEnum = void 0;
  if (userParms.orderBy === undefined) {
    orderByEnum = new _graphql.GraphQLEnumType({
      name: connectionName + 'ConnectionOrder',
      values: {
        ID: { value: ['id', 'ASC'] }
      }
    });
  } else {
    orderByEnum = userParms.orderBy;
  }

  // Assign the connection arguments
  var $connectionArgs = _extends({}, _graphqlRelay.connectionArgs, {
    orderBy: {
      type: new _graphql.GraphQLList(orderByEnum)
    }
  });

  // We are going to give instruction on how
  // the resolver has to retrieve information from
  // rethink, then returning it with in the edges,node pattern.
  var $resolver = require('./../resolver').default(Node, {
    list: true,
    handleConnection: false,
    thinky: Node.thinky,
    before: function before(options, args, context) {
      // We prepare to paginate the result set,
      // because fist or last has been requested
      if (args.first || args.last) {
        options.limit = parseInt(args.first || args.last, 10);
        options.count = true; // count result set

        if (args.after || args.before) {
          var cursor = fromCursor(args.after || args.before);
          var startIndex = Number(cursor.index);

          // Limitation we can't paginate 1 result at the time
          if (startIndex > 0) {
            options.offset = startIndex + 1;
          }
        }
      }

      // attach the order into the composition
      // stack
      var order = void 0;
      if (!args.orderBy) {
        order = [orderByEnum._values[0].value];
      } else if (typeof args.orderBy === 'string') {
        order = [orderByEnum._nameLookup[args.orderBy].value];
      } else {
        order = args.orderBy;
      }

      var orderAttribute = order[0][0]; // Order Attribute
      var orderDirection = order[0][1]; // Order Direction

      // Depending on the direction requested
      // we sort the result accordently
      if (args.last) {
        orderDirection = orderDirection === 'ASC' ? 'DESC' : 'ASC';
      }

      // Assign order
      options.order = [orderAttribute, orderDirection];

      options.relations = [];
      options.attributes = _lodash2.default.uniq(options.attributes);

      return connectionOpts.before(options, args, root, context);
    },
    after: function after(resultset, args, root, _ref) {
      var source = _ref.source;

      var cursor = null;

      // Once we have the result set we decode the cursor
      // if given
      if (args.after || args.before) {
        cursor = fromCursor(args.after || args.before);
      }

      // create edges array
      var edges = resultset.map(function (value, idx) {
        return resolveEdge(value, idx, cursor, args, source);
      });

      var firstEdge = edges[0],
          lastEdge = edges[edges.length - 1];

      var edgeInfo = createEdgeInfo(resultset, args.first || args.last, cursor);
      var hasMorePages = edgeInfo.hasMorePages;
      var hasPreviousPage = edgeInfo.hasPreviousPage;


      return {
        source: source,
        args: args,
        edges: edges,
        pageInfo: {
          startCursor: firstEdge ? firstEdge.cursor : null,
          endCursor: lastEdge ? lastEdge.cursor : null,
          hasPreviousPage: hasPreviousPage,
          hasNextPage: hasMorePages
        }
      };
    }
  });

  // Create a wrapper around our custom resolver
  // So that it will be executed only if edges are
  // returned.
  var resolver = function resolver(source, args, context, info) {
    if ((0, _simplifyAst2.default)(info.fieldASTs[0], info).fields.edges) {
      return $resolver(source, args, context, info);
    }

    return {
      source: source,
      args: args
    };
  };

  resolver.$Node = $resolver.$Node;
  resolver.$before = $resolver.$before;
  resolver.$after = $resolver.$after;
  resolver.$options = $resolver.$options;

  return {
    connectionType: connectionType,
    edgeType: edgeType,
    nodeType: nodeType,
    resolveEdge: resolveEdge,
    connectionArgs: $connectionArgs,
    resolve: resolver
  };
};