'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = resolver;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _thinkyExportSchema = require('thinky-export-schema');

var _thinkyExportSchema2 = _interopRequireDefault(_thinkyExportSchema);

var _graphql = require('graphql');

var _simplifyAst = require('./simplifyAst');

var _simplifyAst2 = _interopRequireDefault(_simplifyAst);

var _generateTree = require('./generateTree');

var _generateTree2 = _interopRequireDefault(_generateTree);

var _queryBuilder = require('./queryBuilder');

var _relay = require('./relay');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

/**
 * Resolver Constructor
 *
 * @param Node
 * @param opts
 * @returns {Resolver}
 */
function resolver(Node) {
  var _this = this;

  var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  if (opts.before === undefined) {
    opts.before = function (opts) {
      return opts;
    };
  }
  if (opts.after === undefined) {
    opts.after = function (opts) {
      return opts;
    };
  }

  var Model = Node.getModel();

  var modelSchema = (0, _thinkyExportSchema2.default)(Model);
  var modelRelations = Object.keys(modelSchema.relationships);

  /**
   * Resolver GraphQL
   *
   * @param source
   * @param args
   * @param context
   * @param info
   * @constructor
   */
  var Resolver = function () {
    var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee(source, args, context, info) {
      var simplyAST, findOptions, nodeArgs, fields, type, result;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              Node.name = Node.name || info.fieldName;

              simplyAST = (0, _simplifyAst2.default)(info.fieldASTs[0], info);
              findOptions = (0, _queryBuilder.argsToFindOptions)(args, Model);
              nodeArgs = _extends({
                attributes: [],
                list: true,
                thinky: null,
                filter: {},
                limit: undefined,
                skip: undefined,
                order: undefined
              }, findOptions);
              fields = simplyAST.fields;
              type = info.returnType;


              if ((0, _relay.isConnection)(info.returnType)) {
                simplyAST = (0, _relay.nodeAST)(simplyAST);
                fields = simplyAST.fields;
                type = (0, _relay.nodeType)(type);
              }

              nodeArgs = opts.before(nodeArgs, args, context, _extends({}, info, {
                ast: simplyAST,
                type: type,
                source: source
              }));

              nodeArgs.thinky = opts.thinky;
              nodeArgs.list = opts.list || type instanceof _graphql.GraphQLList;
              nodeArgs.attributes = Object.keys(fields).filter(function (field) {
                return modelRelations.indexOf(field) === -1 && modelSchema.fields.hasOwnProperty(field) && modelSchema.fields[field] !== 'Virtual';
              }).concat(['id']);

              type = type.ofType || type;

              if (!Node.isRelated()) {
                (function () {
                  var tree = (0, _generateTree2.default)(simplyAST, type, context);

                  Node.setTree(tree);

                  Object.keys(tree).forEach(function (relationName) {
                    var relation = tree[relationName].related;
                    nodeArgs.attributes.push(relation.leftKey);
                  });
                })();
              }

              nodeArgs.attributes = _lodash2.default.uniq(nodeArgs.attributes);
              Node.appendArgs(nodeArgs);

              _context.next = 17;
              return Node.generateDataTree(source, opts.thinky);

            case 17:
              result = _context.sent;
              return _context.abrupt('return', opts.after(result, args, context, _extends({}, info, {
                ast: simplyAST,
                type: type,
                source: source
              })));

            case 19:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, _this);
    }));

    return function Resolver(_x2, _x3, _x4, _x5) {
      return ref.apply(this, arguments);
    };
  }();

  Resolver.$Node = Node;
  Resolver.$before = opts.before;
  Resolver.$after = opts.after;
  Resolver.$options = opts;

  return Resolver;
}